@@ -16,6 +16,7 @@ import ConfigParser
 import types
 import logging
 import shutil
+from string import Template
 logging.basicConfig(level=logging.DEBUG)
 
 try:
@@ -93,6 +94,10 @@ class bigjob_agent:
         # initialization of coordination and communication subsystem
         # Redis initialization
         self.base_url = args[2]
+        self.cds_queue_url = None
+        if len(args)==4:
+            self.cds_queue_url = args[3]
+        logger.debug("External queue: " + str(self.cds_queue_url))
         self.id = self.__get_bj_id(self.base_url)
         logger.debug("BigJob Agent arguments: " + str(args))
         logger.debug("Initialize C&C subsystem to pilot-url: " + self.base_url)
@@ -303,10 +308,12 @@ class bigjob_agent:
                             logger.debug(envi) 
                 
                 executable = job_dict["Executable"]
+                executable = self.__expand_directory(executable)
                 
                 workingdirectory = os.path.join(os.getcwd(), job_id)  
                 if (job_dict.has_key("WorkingDirectory") == True):
                         workingdirectory =  job_dict["WorkingDirectory"]
+                        workingdirectory = self.__expand_directory(workingdirectory)
                 try:
                     os.makedirs(workingdirectory)
                 except:
@@ -551,8 +558,13 @@ class bigjob_agent:
             job_url=self.coordination.dequeue_job(self.base_url)
             logger.debug("Dequed:%s"%str(job_url))
             if job_url==None:
-                time.sleep(3)
-                continue
+                if self.cds_queue_url!=None:
+                    logger.debug("Dequeue sub-job from: " + self.cds_queue_url)       
+                    job_url=self.coordination.dequeue_job(self.cds_queue_url)
+                    logger.debug("Dequed:%s"%str(job_url))
+                if job_url==None:
+                    time.sleep(3)
+                    continue
             if job_url=="STOP":
                 break
             
@@ -680,6 +692,25 @@ class bigjob_agent:
         self.stop=True
         
     
+    def __expand_directory(self, directory):
+        """ expands directory name $HOME or ~ to the working directory
+            on the respective machine 
+        """
+        try:
+            if directory.startswith("$HOME"):
+                template = Template(directory)
+                directory = template.safe_substitute(HOME="~")
+            
+            expanded_directory=os.path.expanduser(directory)
+            logger.debug("Expanded directory: %s to %s"%(directory, expanded_directory))
+            return expanded_directory
+        except:
+            pass
+        
+        return directory
+
+            
+    
     def __get_launch_method(self, requested_method):
         """ returns desired execution method: ssh, aprun """
         
